package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.CourseWeightDto;
import com.qmth.distributed.print.business.bean.dto.obe.CourseRequirementMatrixDto;
import com.qmth.distributed.print.business.bean.dto.obe.MatrixRequirementDto;
import com.qmth.distributed.print.business.bean.dto.obe.MatrixSubRequirementDto;
import com.qmth.distributed.print.business.bean.dto.report.*;
import com.qmth.distributed.print.business.bean.result.ObeCourseRequirementListResult;
import com.qmth.distributed.print.business.bean.result.ObeCourseWeightResult;
import com.qmth.distributed.print.business.entity.ObeCourseOutline;
import com.qmth.distributed.print.business.entity.ObeCourseRequirementReport;
import com.qmth.distributed.print.business.entity.TRBasicInfo;
import com.qmth.distributed.print.business.mapper.ObeCourseRequirementReportMapper;
import com.qmth.distributed.print.business.service.ObeCourseOutlineService;
import com.qmth.distributed.print.business.service.ObeCourseRequirementReportService;
import com.qmth.distributed.print.business.service.TRBasicInfoService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.util.JacksonUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程毕业要求报表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2024-06-13
 */
@Service
public class ObeCourseRequirementReportServiceImpl extends ServiceImpl<ObeCourseRequirementReportMapper, ObeCourseRequirementReport> implements ObeCourseRequirementReportService {
    private final static Logger log = LoggerFactory.getLogger(ObeCourseRequirementReportServiceImpl.class);

    @Resource
    TRBasicInfoService trBasicInfoService;

    @Resource
    ObeCourseRequirementReportService obeCourseRequirementReportService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    ObeCourseOutlineService obeCourseOutlineService;

    /**
     * 换算课程毕业达成度报表详情
     *
     * @param courseRequirementMatrixDtoList
     * @param obeCourseOutlineId
     * @param examId
     * @return
     */
    @Override
    public Map<String, Object> matrixCourseReportInfo(List<CourseRequirementMatrixDto> courseRequirementMatrixDtoList,
                                                      Long obeCourseOutlineId, Long examId,
                                                      List<ObeCourseRequirementDto> obeCourseRequirementDtoList,
                                                      Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap) {
        Map<String, Object> resultMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(courseRequirementMatrixDtoList)) {
            ObeCourseWeightResult obeCourseWeightResult = trBasicInfoService.findCourseWeightResultRmi(obeCourseOutlineId);
            log.info("obeCourseWeightResult:{}", JacksonUtil.parseJson(obeCourseWeightResult));

            List<CourseWeightDto> courseWeightDtoList = obeCourseWeightResult.getSubmitForm();
            //课程目标指标点集合map
            Map<Long, CourseWeightDto> cultureProgramRequirementIdMap = courseWeightDtoList.stream().collect(Collectors.toMap(CourseWeightDto::getObeCultureProgramRequirementId, Function.identity(), (dto1, dto2) -> dto1));
            Map<Long, List<ObeSubRequirementsDto>> matrixSubRequirementIdMap = new HashMap<>();//课程目标指标点集合-多个目标共用一个指标点
            Map<Long, CourseTargetWebDto> courseTargetWebDtoMap = new HashMap<>();//课程目标集合

            ObeCourseOutline obeCourseOutline = obeCourseOutlineService.getById(obeCourseOutlineId);
            Objects.requireNonNull(obeCourseOutline, "课程大纲为空");
            BasicCourse basicCourse = basicCourseService.getById(obeCourseOutline.getCourseId());
            Objects.requireNonNull(basicCourse, "科目信息为空");
            for (CourseRequirementMatrixDto c : courseRequirementMatrixDtoList) {
                if (Objects.equals(c.getCourseCode(), basicCourse.getCode())) {
                    TRBasicInfo trBasicInfo = trBasicInfoService.queryBasicInfo(obeCourseOutline.getCultureProgramId(), obeCourseOutline.getCourseId(), null);
                    if (Objects.nonNull(trBasicInfo)) {
                        ReportCourseEvaluationResultDto reportCourseEvaluationResultDto = Objects.nonNull(trBasicInfo.getCourseEvaluationResult()) ? JSONObject.parseObject(trBasicInfo.getCourseEvaluationResult(), ReportCourseEvaluationResultDto.class) : null;
                        if (Objects.nonNull(reportCourseEvaluationResultDto)) {
                            List<CourseTargetWebDto> courseTargetWebDtoList = reportCourseEvaluationResultDto.getTargetList();
                            courseTargetWebDtoMap = courseTargetWebDtoList.stream().collect(Collectors.toMap(CourseTargetWebDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));
                        }
                        ObeCourseRequirementDto obeCourseRequirementDto = new ObeCourseRequirementDto(c);
                        obeCourseRequirementDtoList.add(obeCourseRequirementDto);
                        List<MatrixRequirementDto> matrixRequirementDtoList = c.getRequirements();
                        if (!CollectionUtils.isEmpty(matrixRequirementDtoList)) {
                            List<ObeRequirementDto> obeRequirementDtoList = new ArrayList<>();
                            obeCourseRequirementDto.setObeRequirements(obeRequirementDtoList);
                            for (MatrixRequirementDto matrixRequirementDto : matrixRequirementDtoList) {
                                ObeRequirementDto obeRequirementDto = new ObeRequirementDto(matrixRequirementDto);
                                obeRequirementDtoList.add(obeRequirementDto);
                                List<MatrixSubRequirementDto> matrixSubRequirementDtoList = matrixRequirementDto.getSubRequirements();
                                if (!CollectionUtils.isEmpty(matrixSubRequirementDtoList)) {
                                    List<ObeSubRequirementsDto> obeSubRequirementsDtoList = new ArrayList<>();
                                    obeRequirementDto.setObeSubRequirements(obeSubRequirementsDtoList);
                                    for (MatrixSubRequirementDto matrixSubRequirementDto : matrixSubRequirementDtoList) {
                                        ObeSubRequirementsDto obeSubRequirementsDto = new ObeSubRequirementsDto(matrixSubRequirementDto);
                                        obeSubRequirementsDtoList.add(obeSubRequirementsDto);
                                        if (Objects.nonNull(matrixSubRequirementDto.getScale())) {
                                            //多个课程目标关联同一个指标点
                                            if (matrixSubRequirementIdMap.containsKey(matrixSubRequirementDto.getSubRequirementId())) {
                                                List<ObeSubRequirementsDto> obeRequirementsDtoList = matrixSubRequirementIdMap.get(matrixSubRequirementDto.getSubRequirementId());
                                                Map<Long, CourseTargetWebDto> map = obeRequirementsDtoList.stream().collect(Collectors.toMap(ObeSubRequirementsDto::getTargetId, Function.identity(), (dto1, dto2) -> dto1));
                                                CourseWeightDto courseWeightDto = cultureProgramRequirementIdMap.get(matrixSubRequirementDto.getSubRequirementId());
                                                if (Objects.nonNull(courseWeightDto)) {
                                                    CourseTargetWebDto courseTargetWebDto = courseTargetWebDtoMap.get(courseWeightDto.getCourseTargetId());
                                                    if (!map.containsKey(courseTargetWebDto.getTargetId())) {
                                                        obeRequirementsDtoList.add(new ObeSubRequirementsDto(courseTargetWebDto, matrixSubRequirementDto));
                                                    }
                                                }
                                                matrixSubRequirementIdMap.put(matrixSubRequirementDto.getSubRequirementId(), obeRequirementsDtoList);
                                            } else {
                                                CourseWeightDto courseWeightDto = cultureProgramRequirementIdMap.get(matrixSubRequirementDto.getSubRequirementId());
                                                if (Objects.nonNull(courseWeightDto)) {
                                                    CourseTargetWebDto courseTargetWebDto = courseTargetWebDtoMap.get(courseWeightDto.getCourseTargetId());
                                                    matrixSubRequirementIdMap.put(matrixSubRequirementDto.getSubRequirementId(), new ArrayList<>(Arrays.asList(new ObeSubRequirementsDto(courseTargetWebDto, matrixSubRequirementDto))));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        //计算各个课程的指标点达成值
                        if (!CollectionUtils.isEmpty(matrixSubRequirementIdMap)) {
                            for (Map.Entry<Long, List<ObeSubRequirementsDto>> entry : matrixSubRequirementIdMap.entrySet()) {
                                Double evaluationValue = entry.getValue().stream().mapToDouble(x -> Objects.nonNull(x.getEvaluationValue()) ? x.getEvaluationValue().doubleValue() : 0.0d).average().orElse(0.0d);
                                ObeSubRequirementsDto obeSubRequirementsDto = entry.getValue().get(0);
                                BigDecimal bigDecimal = new BigDecimal(evaluationValue).multiply(obeSubRequirementsDto.getScale()).setScale(2, BigDecimal.ROUND_HALF_UP);
                                entry.getValue().stream().peek(x -> x.setMatrixDegree(bigDecimal)).collect(Collectors.toList());
                            }
                        }
                        //计算各个课程的指标点达成值
                        List<ObeRequirementDto> obeRequirementDtoList = obeCourseRequirementDto.getObeRequirements();
                        if (!CollectionUtils.isEmpty(obeRequirementDtoList)) {
                            for (ObeRequirementDto o : obeRequirementDtoList) {
                                List<ObeSubRequirementsDto> obeSubRequirementsDtoList = o.getObeSubRequirements();
                                for (ObeSubRequirementsDto obeSubRequirementsDto : obeSubRequirementsDtoList) {
                                    if (matrixSubRequirementIdMap.containsKey(obeSubRequirementsDto.getSubRequirementId())) {
                                        obeSubRequirementsDto.setMatrixDegree(matrixSubRequirementIdMap.get(obeSubRequirementsDto.getSubRequirementId()).get(0).getMatrixDegree());
                                    }
                                    if (obeSubRequirementsDtoMap.containsKey(obeSubRequirementsDto.getSubRequirementId())) {
                                        List<ObeSubRequirementsDto> obeSubRequirementsDtoListTemp = obeSubRequirementsDtoMap.get(obeSubRequirementsDto.getSubRequirementId());
                                        obeSubRequirementsDtoListTemp.add(obeSubRequirementsDto);
                                        obeSubRequirementsDtoMap.put(obeSubRequirementsDto.getSubRequirementId(), obeSubRequirementsDtoListTemp);
                                    } else {
                                        obeSubRequirementsDtoMap.put(obeSubRequirementsDto.getSubRequirementId(), new ArrayList<>(Arrays.asList(obeSubRequirementsDto)));
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        resultMap.put(SystemConstant.OBE_COURSE_REQUIREMENT_DTO_LIST, obeCourseRequirementDtoList);
        resultMap.put(SystemConstant.OBE_SUB_REQUIREMENT_DTO_MAP, obeSubRequirementsDtoMap);
        return resultMap;
    }

    /**
     * 换算课程毕业达成度报表
     *
     * @param obeCourseRequirementDtoList
     * @param obeSubRequirementsDtoMap
     * @return
     */
    @Override
    public ObeCourseRequirementDto matrixCourseReport(List<ObeCourseRequirementDto> obeCourseRequirementDtoList, Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap) {
        //计算指标和毕业达成度
        ObeCourseRequirementDto obeCourseRequirementResultDto = new ObeCourseRequirementDto();
        if (!CollectionUtils.isEmpty(obeCourseRequirementDtoList)) {
            obeCourseRequirementDtoList = this.matrixCourseReportCommon(obeCourseRequirementDtoList, obeSubRequirementsDtoMap);
            ObeCourseRequirementDto obeCourseRequirementDtoTemp = new ObeCourseRequirementDto();
            obeCourseRequirementDtoTemp = obeCourseRequirementDtoList.get(obeCourseRequirementDtoList.size() - 1);
            BeanUtils.copyProperties(obeCourseRequirementDtoTemp, obeCourseRequirementResultDto);
            obeCourseRequirementResultDto.updateInfo();
        }
        return obeCourseRequirementResultDto;
    }

    /**
     * 换算课程毕业达成度报表共用
     *
     * @param obeCourseRequirementDtoList
     * @param obeSubRequirementsDtoMap
     * @return
     */
    @Override
    public List<ObeCourseRequirementDto> matrixCourseReportCommon(List<ObeCourseRequirementDto> obeCourseRequirementDtoList, Map<Long, List<ObeSubRequirementsDto>> obeSubRequirementsDtoMap) {
        for (ObeCourseRequirementDto o : obeCourseRequirementDtoList) {
            List<ObeRequirementDto> obeRequirementDtoList = o.getObeRequirements();
            for (ObeRequirementDto obeRequirementDto : obeRequirementDtoList) {
                List<ObeSubRequirementsDto> obeSubRequirementsDtoList = obeRequirementDto.getObeSubRequirements();
                for (ObeSubRequirementsDto obeSubRequirementsDto : obeSubRequirementsDtoList) {
                    if (obeSubRequirementsDtoMap.containsKey(obeSubRequirementsDto.getSubRequirementId())) {
                        Double degree = obeSubRequirementsDtoMap.get(obeSubRequirementsDto.getSubRequirementId()).stream().mapToDouble(x -> x.getMatrixDegree().doubleValue()).sum();
                        obeRequirementDto.setMatrixDegree(new BigDecimal(degree).setScale(2, BigDecimal.ROUND_HALF_UP));
                    }
                }
                Double degree = obeRequirementDtoList.stream().filter(x -> x.getMatrixDegree().doubleValue() > 0).mapToDouble(x -> x.getMatrixDegree().doubleValue()).min().orElse(0.0d);
                o.setMatrixDegree(new BigDecimal(degree).setScale(2, BigDecimal.ROUND_HALF_UP));
            }
        }
        return obeCourseRequirementDtoList;
    }

    /**
     * 查找课程毕业达成度报表列表
     *
     * @param iPage
     * @param professionalId
     * @param cultureProgramName
     * @return
     */
    @Override
    public IPage<ObeCourseRequirementListResult> obeCourseRequirementReportList(IPage<Map> iPage, Long professionalId, String cultureProgramName) {
        return this.baseMapper.obeCourseRequirementReportList(iPage, professionalId, cultureProgramName);
    }

    /**
     * 根据培养方案删除报表
     *
     * @param cultureProgramId
     * @return
     */
    @Override
    @Transactional
    public boolean removeReport(Long cultureProgramId) {
        ObeCourseRequirementReport obeCourseRequirementReport = obeCourseRequirementReportService.findByCultureProgramId(cultureProgramId);
        return Objects.nonNull(obeCourseRequirementReport) ? obeCourseRequirementReportService.removeById(obeCourseRequirementReport.getId()) : false;
    }

    /**
     * 根据培养方案查询报表
     *
     * @param cultureProgramId
     * @return
     */
    @Override
    public ObeCourseRequirementReport findByCultureProgramId(Long cultureProgramId) {
        return obeCourseRequirementReportService.getOne(new QueryWrapper<ObeCourseRequirementReport>().lambda().eq(ObeCourseRequirementReport::getCultureProgramId, cultureProgramId));
    }
}
